/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (C) 2024 Arm Ltd.
 */

#ifndef _DT_BINDINGS_RST_SUN55I_A523_R_CCU_H_
#define _DT_BINDINGS_RST_SUN55I_A523_R_CCU_H_

#define RST_BUS_R_TIMER		0
#define RST_BUS_R_TWD		1
#define RST_BUS_R_PWMCTRL	2
#define RST_BUS_R_SPI		3
#define RST_BUS_R_SPINLOCK	4
#define RST_BUS_R_MSGBOX	5
#define RST_BUS_R_UART0		6
#define RST_BUS_R_UART1		7
#define RST_BUS_R_I2C0		8
#define RST_BUS_R_I2C1		9
#define RST_BUS_R_I2C2		10
#define RST_BUS_R_PPU1		11
#define RST_BUS_R_IR_RX		12
#define RST_BUS_R_RTC		13
#define RST_BUS_R_CPUCFG	14

#endif /* _DT_BINDINGS_RST_SUN55I_A523_R_CCU_H_ */
