/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2024 Samsung Electronics Co., Ltd.
 * Author: Michal Wilczynski <m.wilczynski@samsung.com>
 */

#ifndef _DT_BINDINGS_TH1520_RESET_H
#define _DT_BINDINGS_TH1520_RESET_H

#define TH1520_RESET_ID_GPU		0
#define TH1520_RESET_ID_GPU_CLKGEN	1
#define TH1520_RESET_ID_NPU		2
#define TH1520_RESET_ID_WDT0		3
#define TH1520_RESET_ID_WDT1		4

#endif /* _DT_BINDINGS_TH1520_RESET_H */
