/* SPDX-License-Identifier: (GPL-2.0-or-later OR MIT) */
/*
 * Actions Semi S900 SPS
 *
 * Copyright (c) 2018 Linaro Ltd.
 */
#ifndef DT_BINDINGS_POWER_OWL_S900_POWERGATE_H
#define DT_BINDINGS_POWER_OWL_S900_POWERGATE_H

#define S900_PD_GPU_B	0
#define S900_PD_VCE	1
#define S900_PD_SENSOR	2
#define S900_PD_VDE	3
#define S900_PD_HDE	4
#define S900_PD_USB3	5
#define S900_PD_DDR0	6
#define S900_PD_DDR1	7
#define S900_PD_DE	8
#define S900_PD_NAND	9
#define S900_PD_USB2_H0	10
#define S900_PD_USB2_H1	11

#endif
