#ifndef __ASM_CPUCAP_DEFS_H
#define __ASM_CPUCAP_DEFS_H

/* Generated file - do not edit */

#define ARM64_ALWAYS_BOOT                             	0
#define ARM64_ALWAYS_SYSTEM                           	1
#define ARM64_BTI                                     	2
#define ARM64_HAS_32BIT_EL0_DO_NOT_USE                	3
#define ARM64_HAS_32BIT_EL1                           	4
#define ARM64_HAS_ADDRESS_AUTH                        	5
#define ARM64_HAS_ADDRESS_AUTH_ARCH_QARMA3            	6
#define ARM64_HAS_ADDRESS_AUTH_ARCH_QARMA5            	7
#define ARM64_HAS_ADDRESS_AUTH_IMP_DEF                	8
#define ARM64_HAS_AMU_EXTN                            	9
#define ARM64_HAS_ARMv8_4_TTL                         	10
#define ARM64_HAS_CACHE_DIC                           	11
#define ARM64_HAS_CACHE_IDC                           	12
#define ARM64_HAS_CNP                                 	13
#define ARM64_HAS_CRC32                               	14
#define ARM64_HAS_DCPODP                              	15
#define ARM64_HAS_DCPOP                               	16
#define ARM64_HAS_DIT                                 	17
#define ARM64_HAS_E0PD                                	18
#define ARM64_HAS_ECV                                 	19
#define ARM64_HAS_ECV_CNTPOFF                         	20
#define ARM64_HAS_EPAN                                	21
#define ARM64_HAS_EVT                                 	22
#define ARM64_HAS_FPMR                                	23
#define ARM64_HAS_FGT                                 	24
#define ARM64_HAS_FGT2                                	25
#define ARM64_HAS_FPSIMD                              	26
#define ARM64_HAS_GCS                                 	27
#define ARM64_HAS_GENERIC_AUTH                        	28
#define ARM64_HAS_GENERIC_AUTH_ARCH_QARMA3            	29
#define ARM64_HAS_GENERIC_AUTH_ARCH_QARMA5            	30
#define ARM64_HAS_GENERIC_AUTH_IMP_DEF                	31
#define ARM64_HAS_GIC_CPUIF_SYSREGS                   	32
#define ARM64_HAS_GIC_PRIO_MASKING                    	33
#define ARM64_HAS_GIC_PRIO_RELAXED_SYNC               	34
#define ARM64_HAS_HCR_NV1                             	35
#define ARM64_HAS_HCX                                 	36
#define ARM64_HAS_LDAPR                               	37
#define ARM64_HAS_LPA2                                	38
#define ARM64_HAS_LSE_ATOMICS                         	39
#define ARM64_HAS_MOPS                                	40
#define ARM64_HAS_NESTED_VIRT                         	41
#define ARM64_HAS_PAN                                 	42
#define ARM64_HAS_PMUV3                               	43
#define ARM64_HAS_S1PIE                               	44
#define ARM64_HAS_S1POE                               	45
#define ARM64_HAS_RAS_EXTN                            	46
#define ARM64_HAS_RNG                                 	47
#define ARM64_HAS_SB                                  	48
#define ARM64_HAS_STAGE2_FWB                          	49
#define ARM64_HAS_TCR2                                	50
#define ARM64_HAS_TIDCP1                              	51
#define ARM64_HAS_TLB_RANGE                           	52
#define ARM64_HAS_VA52                                	53
#define ARM64_HAS_VIRT_HOST_EXTN                      	54
#define ARM64_HAS_WFXT                                	55
#define ARM64_HAFT                                    	56
#define ARM64_HW_DBM                                  	57
#define ARM64_KVM_HVHE                                	58
#define ARM64_KVM_PROTECTED_MODE                      	59
#define ARM64_MISMATCHED_CACHE_TYPE                   	60
#define ARM64_MPAM                                    	61
#define ARM64_MPAM_HCR                                	62
#define ARM64_MTE                                     	63
#define ARM64_MTE_ASYMM                               	64
#define ARM64_SME                                     	65
#define ARM64_SME_FA64                                	66
#define ARM64_SME2                                    	67
#define ARM64_SPECTRE_V2                              	68
#define ARM64_SPECTRE_V3A                             	69
#define ARM64_SPECTRE_V4                              	70
#define ARM64_SPECTRE_BHB                             	71
#define ARM64_SSBS                                    	72
#define ARM64_SVE                                     	73
#define ARM64_UNMAP_KERNEL_AT_EL0                     	74
#define ARM64_WORKAROUND_834220                       	75
#define ARM64_WORKAROUND_843419                       	76
#define ARM64_WORKAROUND_845719                       	77
#define ARM64_WORKAROUND_858921                       	78
#define ARM64_WORKAROUND_1418040                      	79
#define ARM64_WORKAROUND_1463225                      	80
#define ARM64_WORKAROUND_1508412                      	81
#define ARM64_WORKAROUND_1542419                      	82
#define ARM64_WORKAROUND_1742098                      	83
#define ARM64_WORKAROUND_1902691                      	84
#define ARM64_WORKAROUND_2038923                      	85
#define ARM64_WORKAROUND_2064142                      	86
#define ARM64_WORKAROUND_2077057                      	87
#define ARM64_WORKAROUND_2457168                      	88
#define ARM64_WORKAROUND_2645198                      	89
#define ARM64_WORKAROUND_2658417                      	90
#define ARM64_WORKAROUND_AMPERE_AC03_CPU_38           	91
#define ARM64_WORKAROUND_AMPERE_AC04_CPU_23           	92
#define ARM64_WORKAROUND_TRBE_OVERWRITE_FILL_MODE     	93
#define ARM64_WORKAROUND_TSB_FLUSH_FAILURE            	94
#define ARM64_WORKAROUND_TRBE_WRITE_OUT_OF_RANGE      	95
#define ARM64_WORKAROUND_CAVIUM_23154                 	96
#define ARM64_WORKAROUND_CAVIUM_27456                 	97
#define ARM64_WORKAROUND_CAVIUM_30115                 	98
#define ARM64_WORKAROUND_CAVIUM_TX2_219_PRFM          	99
#define ARM64_WORKAROUND_CAVIUM_TX2_219_TVM           	100
#define ARM64_WORKAROUND_CLEAN_CACHE                  	101
#define ARM64_WORKAROUND_DEVICE_LOAD_ACQUIRE          	102
#define ARM64_WORKAROUND_NVIDIA_CARMEL_CNP            	103
#define ARM64_WORKAROUND_PMUV3_IMPDEF_TRAPS           	104
#define ARM64_WORKAROUND_QCOM_FALKOR_E1003            	105
#define ARM64_WORKAROUND_QCOM_ORYON_CNTVOFF           	106
#define ARM64_WORKAROUND_REPEAT_TLBI                  	107
#define ARM64_WORKAROUND_SPECULATIVE_AT               	108
#define ARM64_WORKAROUND_SPECULATIVE_SSBS             	109
#define ARM64_WORKAROUND_SPECULATIVE_UNPRIV_LOAD      	110
#define ARM64_NCAPS					111

#endif /* __ASM_CPUCAP_DEFS_H */
